#!/usr/bin/env python
# -*- coding:utf-8 -*

#Pour travailler sur les sources
import sys
sys.path.insert(0,'../../FGPIO')
sys.path.insert(0,'../../FUTIL')

from FGPIO.qrd1114_io import *
from FGPIO.mcp300x_hspi_io import *
from FGPIO.led_io import *
from FGPIO.bt_io import *
from FGPIO.i2c_device_io import *

import time
import datetime
import logging

#TODO : créer start_record()
#		gestion HC/HP

class compteur_edf(object):
	'''Un compteur EDF avec une grosse roue qui tourne
		Sur lequel on va pouvoir calculer les consos a l'aide d'un capteur qrd1114 placé devant la roue
		qui va détecter le passage de la bande noire.
	'''
	timeout = 60		#Correspond à 150Watts
	time_pause = 0.001	#duree pause entre chaque mesure
	micro_controleur_RESET = 88		#Ordre de reset du micro controleur
	def __init__(self, capteur = None, micro_controleur = None, capteur_hc = None, led = led_io(), intensity_max=45, energy_tr=2.5, nb_tours_mesure = 1, counter_hc = 0, counter_hp = 0, led_hc = led_io()):
		'''Initialisation
			- capteur			: 	qrd1114_io sur la roue (optionnel) (remplace le micro_controleur)
			- micro_controleur	:	micro controleur pour mesure nb de tours (optionnel) (remplace le capteur optique)
			- capteur_hc		:	capteur d'heure creuse (class bt_io)
			- intensite_max		:	intensité maximum de l'abonnement
			- energy_tr			:	energie par tour de roue en Wh/tr
			- led				:	led pour une impulsion par tour
			- nb_tours_mesure	:	Nb de tour de roue utilisés pour réaliser la mesure
		'''
		if micro_controleur==None:
			assert isinstance(capteur, qrd1114_analog_io), "capteur must be a qrd1114_analog_io"	
		self.capteur = capteur
		if capteur==None:
			assert isinstance(micro_controleur, i2c_device_io), "capteur must be a i2c_device_io"	
		self.micro_controleur = micro_controleur
		assert capteur_hc == None or isinstance(capteur_hc, bt_io), "capteur_hc must be a bt_io"
		self.capteur_hc = capteur_hc
		assert isinstance(led, led_io), "led must be a led_io"
		self.led = led
		self.energy_tr = energy_tr
		self.mini_tr = 3600. / (intensity_max * 230)*self.energy_tr
		self.mini_noir = self.mini_tr *2.5/100*0.4 # durée mini du trait noir qui correspond à 2.5% du cercle -60% de marge
		self.nb_tours_mesure = nb_tours_mesure
		self.seuil_delestage = intensity_max * 230 * 0.90 # à 90% du max
		self.counter_hc = counter_hc
		self.counter_hp = counter_hp
		self.terminated = False #Pour tuer l'enregistrement
		self.led_hc = led_hc
		# Creation d'une thread pour gestion led heure creuses
		if capteur_hc:
			self.capteur_hc.add_thread(on_changed = self.on_capteur_hc_change, pause = 1)
		

	def init_params(self):
		''' Calcul les paramètres de détection
		'''
		calculated = False
		while not calculated:
			logging.info("Mesure des paramètres du capteur......")
			self.led.blink(0.2)
			moyenne = 0
			maxi = 0
			nb = 0
			fin = time.time() + self.timeout
			while (maxi < moyenne * 5  or nb < 100) and time.time() < fin:
				value = self.capteur.read()
				moyenne = moyenne*nb+value
				nb+=1
				moyenne /= nb
				if value > maxi:
					maxi = value
					logging.debug("Voltage maxi : %s" % maxi)
				time.sleep(self.time_pause)
			self.voltage_moyen = moyenne
			self.gain_seuil = 1 + (maxi / moyenne - 1) * 0.50	# On se laisse une petite marge de 50%
			self.led.stop()
			if self.gain_seuil < 1.25:
				logging.warning("Seuil de detection de bande noire insuffisant.")
			else:
				logging.info( "Voltage_moyen : %.4f - Voltage maxi : %.4f - Seuil : %.4f"%(self.voltage_moyen, maxi, self.voltage_moyen*self.gain_seuil))
				calculated = True
	
	
	def init_record(self):
		'''Doit être lancée avant la première mesure
		'''
		if self.capteur:
			#Si capteur optique
			logging.info("Initialisation du capteur optique.")
			self.init_params()
			self.wait_for_black()
			logging.info("Initialisation du capteur optique ok")
		elif self.micro_controleur:
			#Si micro controleur
			logging.info("Initialisation du micro_controleur...")
			connection = False
			while not connection:
				try:
					self.micro_controleur.device.write_cmd(self.micro_controleur_RESET) 
					# Attente de la reponse du micro_controleur
					while self.micro_controleur.device.read()==0:
						time.sleep(1)
					connection = True
				except IOError:
					logging.error("Connection au micro_controleur impossible. Retry in 10 seconds.")
					time.sleep(10)
			logging.info("Initialisation du micro_controleur ok.")
			
	def record(self, rec_function):
		'''Boucle infinie pour mesures et enregistrement
			- rec_function		:		function qui va prendre les donnees en entrée
		'''
		if self.capteur:
			#Si capteur optique
			self.record_optique(rec_function)
		elif self.micro_controleur:
			#Si micro controleur pour lecture nb de tours
			self.record_micro_controleur(rec_function)
		
	
	def record_capteur(self, rec_function):
		'''Boucle infinie pour mesures et enregistrement
			- rec_function		:		function qui va prendre les donnees en entrée
		'''
		logging.debug('compteur_edf.record() start.')
		_power = 0
		i = 0
		while i<self.nb_tours_mesure and not self.terminated:
			i+=1
			now = time.time()
			self.led.on()
			time.sleep(self.mini_tr/2)
			self.led.off()
			self.wait_for_black()
			duree = time.time() - now
			_power += self.energy_tr*3600/duree
		if not self.terminated:
			logging.debug('%s tour enregistres.' % self.nb_tours_mesure)
			# Mise à jours des compteurs (HC et HP)
			if self.type_horaire() == 'HC':
				self.counter_hc += self.energy_tr * self.nb_tours_mesure
				logging.info('Compteur HC = %s.' % self.counter_hc)
			else:
				self.counter_hp += self.energy_tr * self.nb_tours_mesure
				logging.info('Compteur HP = %s.' % self.counter_hp)
			# Enregistrement
			rec_function( \
					date_time=datetime.datetime.now(), \
					energy = self.energy_tr * self.nb_tours_mesure, \
					power = int(_power/self.nb_tours_mesure))
	
	def record_micro_controleur(self, rec_function):
		'''Boucle infinie pour mesures et enregistrement
			- rec_function		:		function qui va prendre les donnees en entrée
		'''
		now = time.time()
		logging.debug('compteur_edf.record() start.')			
		nb_tours_enregistres = 0
		while nb_tours_enregistres < self.nb_tours_mesure and not self.terminated:
			nb_tours_lu = self.micro_controleur.device.read()
			if nb_tours_lu > 0:
				self.led.on()
				logging.debug('Lecture du micro_controleur : %s tour(s)' % nb_tours_lu)
				nb_tours_enregistres+=nb_tours_lu
			time.sleep(self.mini_tr)
			self.led.off()
		if not self.terminated:
			logging.debug('%s tour enregistres.' % nb_tours_enregistres)
			# Calcul Energie et puissance
			duree = time.time() - now
			energy = self.energy_tr * nb_tours_enregistres
			power = energy * 3600 / duree
			# Mise à jours des compteurs (HC et HP)
			if self.type_horaire() == 'HC':
				self.counter_hc += energy
				logging.info('Compteur HC = %s.' % self.counter_hc)
			else:
				self.counter_hp += energy
				logging.info('Compteur HP = %s.' % self.counter_hp)
			# Enregistrement
			rec_function( \
				date_time=datetime.datetime.now(), \
				energy = energy, \
				power = int(power))
			
			
	def stop_record(self):
		'''Stop the record fonction
		'''
		self.terminated = True
		self.capteur_hc.stop()
	
	def wait_for_black(self):
		''' Attends que le voltage augmente jusqu'au voltage_seuil pendant au moins self.mini_noir ms
			Si ça dure trop longtemp, relance la mesure de la moyenne
		'''
		logging.debug('compteur_edf.wait_for_black().')
		timeout = time.time() + self.timeout
		black = False
		seuil=self.voltage_moyen*self.gain_seuil
		duree_noire = 0
		while not black:
			while duree_noire < self.mini_noir and time.time()<timeout:
				now=time.time()
				maximum_voltage = 0
				voltage = self.capteur.read()
				while voltage > seuil and time.time()<timeout:
					voltage = self.capteur.read() # temps de lecture du capteur < 0.0004 secondes
					maximum_voltage = max(maximum_voltage, voltage)
					time.sleep(self.time_pause/2)
				if maximum_voltage>0: # Pour gerer le cas ou le processeur fait autre chose!
					duree_noire = time.time()-now
					if duree_noire<self.mini_noir:
						logging.debug("Fausse bande noire : % secondes. Voltage maximum : %s" % (duree_noire, maximum_voltage))
				else:
					duree_noire = 0
				time.sleep(self.time_pause)
			if time.time()>timeout :
				logging.warning("Perte de la bande noire")
				self.init_params()
				timeout = time.time() + self.timeout
			else:
				black = True
				logging.debug("Durée de la bande noire : %s. Voltage maximum : %s" % (duree_noire, maximum_voltage))
	
				
	def mesure_moy(self, duration = 1):
		moyenne = 0
		nb = 0
		now=time.time()
		while time.time()<now+duration:
			moyenne += self.capteur.read()
			nb += 1
		return moyenne / nb
	
	def type_horaire(self):
		'''Renvoie le type d'horaire HC/HP
		'''
		if self.capteur_hc.read():
			return 'HC'
		else:
			return 'HP'
	
	def on_capteur_hc_change(self):
		''' Methode executer en cas de changement d'état HC/HP
		'''
		if self.capteur_hc.read():
			self.led_hc.on()
			logging.info("Période HEURES CREUSES activée.")
		else:
			self.led_hc.off()
			logging.info("Période HEURES CREUSES Désactivée.")

#########################################################
#                                                       #
#		EXEMPLE                                         #
#                                                       #
#########################################################

if __name__ == '__main__':
	pc = rpiduino_io()
	mcp3008 = mcp3008_hspi_io()	#Pour lecture analogique sur Rpi
	compteur = compteur_edf(qrd1114_analog_io(mcp3008.pin[0]), led_io(pc.bcm_pin(16)), 45,2.5)
	def rec_f(date_time, energy, power, counter):
			print "%s : %.1fWh ie %.1fWatts. Compteur : %s"%(date_time, energy, power, counter)
	compteur.record(rec_f)
	
	


#!/usr/bin/env python
# -*- coding:utf-8 -*

'''
	Programme principal de comptage electrique
	et enregistrement dans base de données
	
	la config se trouve dans 
			config.py
'''

#Pour travailler sur les sources
import sys
sys.path.insert(0,'../FUTIL')
import time

from FUTIL.my_logging import *

my_logging(console_level = DEBUG, logfile_level = INFO, details = False)

import config

logging.info('comptage.py démarre')

home = config.get_installation()

home.start_comptage_secondaire()
home.start_comptage_general()

print 'press CTRL-C to stop process'

try: #Ca permet de pouvoir planter le thread avec un CTRL-C
	while True:
		time.sleep(1)
except KeyboardInterrupt:
	home.stop_comptage_secondaire()
	home.stop_comptage_general()



#!/usr/bin/env python
# -*- coding:utf-8 -*

####################################
'''
Lecture de la base de données des températures
Et envoie de mail si alerte.

Plus création des conso déduites.
'''
#################################### 

#Pour travailler sur les sources
import sys
sys.path.insert(0,'../FGPIO')
sys.path.insert(0,'../FUTIL')

import FERG.tempeDB
import FERG.installation
from FUTIL.my_logging import *
import FUTIL.mails

#TODO : A sécuriser : + redondant avec config.py 
MyMail = FUTIL.mails.gmail(gmail_account='xxx', gmail_pwd='xxx')


my_logging(console_level = INFO, logfile_level = INFO)

logging.info("Execution de chk_alertes.py")


# Gestion des températures
db = FERG.tempeDB.tempeDB(db_name="tempeDB", user="fred", passwd="achanger", host="192.168.10.10", mail = MyMail)
db.check_alertes()
db.chk_mesures()

#Gestion des consos electriques
import config
install = config.get_installation(physical_device = False)

install.deduit_conso_arithmetic()
install.deduit_conso_typiques()
install.check_alertes()



#!/usr/bin/env python
# -*- coding:utf-8 -*
import itertools
import logging

class circuit():
	''' Un circuit électrique
	'''
	def __init__(self, nom, parent=None, puissance_maxi = None, energie_maxi_jour = None, delest = None, puissance_typique = None, fonctionement_max = None, max_average_power = None, duration_average_power = 3600):
		'''Initialisation
			- nom					:	nom du circuit
			- parent				:	circuit parent
			- puissance_maxi		:	puissance maximal (non utilise pour l'instant) TODO : alertes
			- energie_maxi_jour		:	energie_maxi_jour (non utilise pour l'instant) TODO : alertes
			- delest				:	relay_io qui peut couper le circuit quand l'installation totale est en delestage
			- puissance_typique		:	puissance quand l'apareil est en fonctionnement (utilisé pour déduire des consommation ou gérer le delestage)
			- fonctionement_max		:	temps de fonctionnement maximum (en secondes) de l'apareil à > 50% de la puissance typique. Gère une alerte.
			- max_average_power		:	puissance moyenne maximum. Gère une alerte.
			- duration_average_power:	temps en secondes pour calcul max_average_power
		'''
		self.nom = nom
		self.parent = parent
		self.puissance_maxi = puissance_maxi
		self.energie_maxi_jour = energie_maxi_jour
		self.enfants = []
		self.delest = delest
		self.puissance_typique = puissance_typique
		self.fonctionement_max = fonctionement_max
		self.max_average_power = max_average_power
		self.duration_average_power = duration_average_power
		
	# def __repr__(self):
		# repr = self.nom + "\n"
		# for circuit in self.enfants:
			# repr += "|__%s\n" % circuit
		# return repr
				
class circuit_mesure(circuit):
	'''Un circuit mesuré
	'''
	def __init__(self,  nom, compteur, parent=None, eclatable = False, puissance_maxi= None, energie_maxi_jour= None, delest = None, puissance_typique = None, fonctionement_max = None, max_average_power = None, duration_average_power = 3600):
		'''Initialisation
		'''
		self.eclatable = eclatable
		self.compteur = compteur
		circuit.__init__(self, nom, parent, puissance_maxi, energie_maxi_jour, delest, puissance_typique, fonctionement_max, max_average_power, duration_average_power)
	
	def __repr__(self):
		return "Circuit_mesure : %s" % self.nom
	
	def is_deductible_typically(self):
		'''renvoie True si
			est eclatagle et
			tous les circuits enfants ont des consos typiques
			et les valeurs de conso typiques 'vont bien' pour déduire
		'''
		if not self.eclatable:
			return False
		for enfant in self.enfants:
			try:
				if enfant.puissance_typique in (0, None) :
					return False
			except AttributeError:
				return False
		combinaisons = self.combinaisons_enfants_typiques()
		for i in range(0, len(combinaisons)):
			for j in range(i+1, len(combinaisons)):
				ratio = 1.*self.comb_sum(combinaisons[i])/self.comb_sum(combinaisons[j])
				if ratio>0.8 and ratio < 1.2: #Conditions : 80% - 120%
					return False
		return True
	
	def is_deductible_arithmetically(self):
		'''Renvoie un tuple de 2 elements avec 
				- le circuit calculable
				- les circuits à soustraire
			Si
				- est eclatable
				- il n'a y qu'un seul sous-circuit non mesuré
			Sinon, renvoie None
		'''
		if self.eclatable:
			circuits_calculable = []
			circuits_a_soustraire = []
			for enfant in self.enfants:
				if isinstance(enfant, circuit_mesure):
					circuits_a_soustraire.append(enfant)
				else:
					circuits_calculable.append(enfant)
			if len(circuits_calculable) == 1:
				return circuits_calculable[0], circuits_a_soustraire
	
	def combinaisons_enfants_typiques(self):
		'''renvoie les combinaisons possibles de circuits
		'''
		combinaisons = []
		for i in range(0,len(self.enfants)+1):
			for combinaison in itertools.combinations(self.enfants, i):
				combinaisons.append(combinaison)
		return combinaisons
	
	@staticmethod
	def comb_sum(combinaison):
		'''calcul la somme des conso typiques d'une liste de circuit
		'''
		somme = 0
		for circuit in combinaison:
			somme += circuit.puissance_typique
		return somme
		
class circuit_general(circuit_mesure):
	'''Circuit général
	'''
	def __init__(self,  nom, compteur = None, eclatable = False, puissance_maxi= None, energie_maxi_jour= None):
		'''Initialisation
		'''
		circuit_mesure.__init__(self, nom, compteur, None, eclatable, puissance_maxi, energie_maxi_jour, None)
	
	def __repr__(self):
		return "Circuit_général : %s" % self.nom

class electric_device(circuit):
	'''Un circuit physique mais non mesuré
	'''
	def __repr__(self):
		return "Circuit_device : %s" % self.nom


#!/usr/bin/env python
# -*- coding:utf-8 -*

####################################
'''
Enregistrement sur une Base de donnée 
des températures mesurés et transmise par RF
'''
#################################### 
#Pour travailler sur les sources
import sys
sys.path.insert(0,'../../FGPIO')
sys.path.insert(0,'../../FUTIL')

from FGPIO.rf_recept_io import *
from FERG.tempeDB import *
import datetime
from FUTIL.my_logging import *
import json

my_logging(console_level = DEBUG, logfile_level = DEBUG)

logging.info('rf_temp_io démarre')

db=tempeDB()

f_json = "rf_temp_io.json"
valeurs = {}

pc = rpiduino_io()
rf = temperature_rf_recept_io(pc.bcm_pin(4))

while True:
	capteurs = db.get_capteurs()
	if capteurs:
		for capteur in capteurs:
			if capteur[0] in rf.sensors:
				if rf.sensors[capteur[0]]!=capteur[1]:
					rf.sensors[capteur[0]]=capteur[1]
					logging.info("Nom du capteur %s modifié de %s à %s." % (capteur[0], rf.sensors[capteur[0]],capteur[1]))
			else:
				rf.add_sensor(capteur[0], capteur[1])
				logging.info("Capteur %s : %s ajoute" % (capteur[0:2]))
			logging.debug("Mesure de %s ..." % capteur[1])
			valeur = rf.read(capteur[1])
			logging.info("Mesure de %s : %s." % (capteur[1], valeur))
			if valeur != None :
				if capteur[2] != None and capteur[2] != '':
					try:
						valeur = eval(capteur[2])
					except Exception, e:
						logging.error("Error in function of the sensor : '%s'. %e" % (capteur[2], e))
						valeur = None
			if valeur != None:
				valeur = round(valeur,2)
				valeurs[capteur[1]]=valeur
				with open(f_json,"w") as f:
					json.dump(valeurs, f)
				db.add(capteur[1], datetime.datetime.now(), valeur)
	time.sleep(60)




#!/usr/bin/env python
# -*- coding:utf-8 -*

#Pour travailler sur les sources
import sys
sys.path.insert(0,'../../FGPIO')
sys.path.insert(0,'../../FUTIL')

import mysql.connector
import time
import functools
import threading
import datetime
import logging
from circuit import *
from FUTIL.mails import *


class DB:
	""" Classe pour une base de données pour enregistrement données"""
	def __init__(self, db_name="tempeDB", user="fred", passwd="achanger", host="192.168.10.10", mail = None):
		'''Initialisation
			- db_name		:	nom de la base de données
			- user			:	user de la base de données
			- passwd		:	password de la base de données
			- host			:	host de la base de données
			- mail			:	FUTIL.mails.gmail
		'''
		self.db_name = db_name
		self.user = user
		self.passwd = passwd
		self.host = host
		self.db = None
		self.connect()
		self.smtp = mail
		self._sql_reqs = [] # Stock les requetes actions (INSERT) avant de les executer
		self.lock = threading.Lock()
	
				
	def connect(self):
		'''Connecte the database
		'''
		try:
			self.db =  mysql.connector.connect(database=self.db_name, user = self.user, password = self.passwd, host = self.host)
		except mysql.connector.errors.Error, e:
			logging.error('mysql Reconnection failed. ' + str(e))
	
	@staticmethod
	def unError(fonction):
		'''Decorateur pour eviter les erreurs
		Si erreur dans une des methodes, on essaye de reconnecter la base de données 1 fois
		Si ca ne passe toujours pas, on abandonne'''
		@functools.wraps(fonction) # ca sert pour avoir un help(SBClient) utile
		def UnErrorFonction(self,*args, **kwargs):
			try:
				return fonction(self,*args, **kwargs)
			except mysql.connector.errors.Error as e: # pour uniquement mysql : mysql.connector.errors.Error
				logging.warning(repr(e) + ' in ' + fonction.__name__)
				logging.warning("Reconnection en cours ...")
				self.connect()
				try:
					return fonction(self,*args, **kwargs)
				except Exception, e:
					logging.error('Reconnection failed. ' + str(e))
		return UnErrorFonction
		
		
	def chk_mesures(self, nom = None):
		''' Vérifie que les mesures sont bien réalisées
			Envoie un email cas échéant
		'''
		cursor = self.db.cursor()
		req = "SELECT id, nom, table_donnees, duree, message, contact, duree_repetition, date_envoie FROM grandeur_mesuree WHERE test_enregistrement = 1"
		if nom != None:
			req += " AND nom = %s"
			cursor.execute(req,(nom))
		else:
			cursor.execute(req)
		grandeur_mesurees = cursor.fetchall()
		for grandeur_mesuree in grandeur_mesurees:
			[alerte_id, nom, table_donnees, duree, message, contact, duree_repetition, date_envoie] = grandeur_mesuree
			duree = datetime.timedelta(hours = float(duree))
			if table_donnees == 'consos_electrique':
				champ_date = 'date_fin'
				champ_mesure = 'circuit'
			else:
				champ_date = 'date'
				champ_mesure = 'capteur'
			# Pour éviter le risque injection SQL sur table_donnees, on limite à 40 caracteres
			table_donnees = table_donnees[:40]
			req = "SELECT " + champ_date + " FROM "+ table_donnees +" WHERE " + champ_mesure + " = %s AND " + champ_date + " = (SELECT MAX(" + champ_date + ") FROM "+ table_donnees +" WHERE " + champ_mesure + " = %s)"
			cursor.execute(req,(nom, nom))
			derniere_mesure = cursor.fetchone()
			if derniere_mesure != None:
				derniere_mesure = derniere_mesure[0]
				test =  datetime.datetime.now() - derniere_mesure > duree
				if test and (date_envoie==None or date_envoie + datetime.timedelta(hours=duree_repetition) < datetime.datetime.now()):
					if '%s' in message:
						message = message % derniere_mesure
					req = "SELECT email FROM contacts WHERE nom = '%s'" % (contact)
					cursor.execute(req)
					email = cursor.fetchone()
					if self.smtp.send_mail(email[0], u'Alerte enregistrement', message):
						req = "UPDATE grandeur_mesuree SET date_envoie = %s WHERE id = %s"
						cursor.execute(req,(datetime.datetime.now(), alerte_id))
						self.db.commit()

class DB_buffered(DB):
	'''Juste pour que l'héritage de @DB.unError fonctionne
	'''
	@DB.unError
	def _action_sql_req(self):
		''' Execute the request in _sql_reqs on database.
		'''
		cursor = self.db.cursor()
		while self._sql_reqs:
			req = self._sql_reqs[0][0]
			args = self._sql_reqs[0][1]
			logging.debug('SQL==> "%s"%s' % (req, args))
			cursor.execute(req, args)
			self.db.commit()
			del self._sql_reqs[0]
			logging.debug('SQL=> ok')

	def action_sql_req(self, req, *args):
		'''Add the request in _sql_reqs
			and run _action_sql_req
			- req		:	sql resquest
			- *args		:	les arguments de la requetes (qui vont avec  %s)
		'''
		with self.lock:
			self._sql_reqs.append([req,args])
			self._action_sql_req()

		
class tempeDB(DB_buffered):
	""" Classe pour une base de données pour enregistrement de la temperature"""
#	def __init__(self, db_name="tempeDB", user="fred", passwd="achanger", host="192.168.10.10"):
#		DB.__init__(self,db_name, user, passwd, host)
	
	@DB.unError	
	def add(self, capteur, date, temperature):
		""" INSERT sur la table "mesures" une date et une température"""
		(min, max) = self.get_min_max(capteur)
		if min < temperature < max:
			req = "INSERT INTO mesures (capteur, date, temperature) VALUES ( '%s', '%s' , %s)" % (capteur, date.strftime('%Y-%m-%d %H:%M:%S'), temperature)
			self.action_sql_req(req)
		else:
			logging.error('Temperature (%s) out of range : %s (must be between %s and %s)' % (capteur, temperature, min, max))
	
	def get_min_max(self, capteur):
		'''Renvoie un tuple (min, max) représentant les valeurs possibles mini et maxi de la mesure
		'''
		try:
			req = "SELECT min, max FROM capteurs WHERE nom = '%s'" % capteur
			cursor = self.db.cursor()
			cursor.execute(req)
			result = cursor.fetchone()
			logging.debug("get_min_max : result = " + str(result))
			(a,b) = result
		except:
			return (-99999,99999)
		else:
			return result
	
	@DB.unError
	def last_temperature(self, capteur=None):
		'''Return the last temperature 
		'''
		if capteur == None:
			capteurs = [n[1] for n in self.get_capteurs()]
		else:
			capteurs = [capteur]
		result = {}
		cursor = self.db.cursor()
		for capt in capteurs:
			req = "SELECT  temperature FROM mesures WHERE capteur=%s AND date = (SELECT MAX(date) FROM mesures WHERE capteur=%s)"
			cursor.execute(req,(capt, capt))
			temperature = cursor.fetchone()
			if temperature != None:
				result[capt] = float(temperature[0])
		if capteur != None:
			try:
				return result[capteur]
			except KeyError:
				return None
		else:
			return result
			
	@DB.unError			
	def check_alertes(self):
		'''Vérifie la tables des alertes 
			et envoie un mail si besoin.
		'''
		cursor = self.db.cursor()
		req = "SELECT `id`,`capteur`,`min_max`,`valeur`,`message`,`contact`,`duree_repetition`,`date_envoie` FROM `alertes`";
		cursor.execute(req)
		alertes = cursor.fetchall()
		for alerte in alertes:
			[alerte_id, capteur, min_max, valeur, message, contact, duree_repetition, date_envoie] = alerte
			req = "SELECT temperature FROM mesures WHERE capteur=%s AND date = (SELECT MAX(date) FROM mesures WHERE capteur=%s)"
			cursor.execute(req,(capteur, capteur))
			temperature = cursor.fetchone()
			if temperature != None:
				temperature = temperature[0]
				if min_max=='MIN':
					test = temperature < valeur
				else:
					test = temperature > valeur
				if test and (date_envoie==None or date_envoie + datetime.timedelta(hours=duree_repetition) < datetime.datetime.now()):
					if '%s' in message:
						message = message % temperature
					req = "SELECT email from contacts where nom = '%s'" % contact
					cursor.execute(req)
					email = cursor.fetchall()
					if self.smtp.send_mail(email[0][0], u'Alerte températures', message):
						req = "UPDATE alertes SET date_envoie = '%s' WHERE id = '%s'" % (datetime.datetime.now(), alerte_id)
						cursor.execute(req)
						self.db.commit()
	
	@DB.unError					
	def get_capteurs(self):
		'''Renvoie la liste des capteurs
		'''
		cursor = self.db.cursor()
		req = "SELECT `id`, `nom`, `formule` FROM `capteurs`"
		logging.debug('SQL==> '+req)
		cursor.execute(req)
		result = cursor.fetchall()
		if isinstance(result, list):
			return result
		else:
			return []
	

class conso_electriqueDB(DB_buffered):
	''' Classe pour une base de données pour enregistrement des consos electriques
	'''
	@DB.unError
	def add(self, circuit, date_debut, date_fin, energie, puissance, type_horaire=''):
		""" INSERT sur la table "consos_electrique" les infos de consommation"""
		req = "INSERT INTO consos_electrique (circuit, date_debut, date_fin, energie, puissance, type_horaire) VALUES (%s, %s, %s , %s, %s, %s)"
		self.action_sql_req(req, circuit, date_debut.strftime('%Y-%m-%d %H:%M:%S'), date_fin.strftime('%Y-%m-%d %H:%M:%S'), energie, puissance, type_horaire)
	
	# OBSOLETE!!!
	@DB.unError
	def _get_last_counter(self, circuit):
		''' recherche la dernière valeur du compteur
		'''
		cursor = self.db.cursor()
		req = "SELECT  compteur  FROM consos_electrique WHERE circuit = %s AND date_debut = (SELECT MAX(date_debut) FROM  consos_electrique WHERE circuit = %s)"
		logging.debug('SQL==> ' + req + str((circuit, circuit)))
		cursor.execute(req, (circuit, circuit))
		result = cursor.fetchone()
		cursor.close()
		if result and result[0]:
			result = float(result[0])
		else:
			result = 0
		return result
	
	def get_last_counter(self, circuit):
		''' recherche la dernière valeur du compteur
		'''
		result = self._get_last_counter(circuit)
		if not result:
			result = 0
		logging.debug('get_last_counter for %s : %s' % (circuit, result))
		return result
	
	@DB.unError
	def eclate_circuit(self):
		'''Analyse les consommations et éclate les valeurs sur des sous-circuits ou electric_device
		'''
		pass
		
	@DB.unError
	def get_circuits_mesures(self):
		'''Extrait les circuits
		'''
		cursor = self.db.cursor()
		req = "SELECT  nom, parent, eclatable, puissance_maxi, energie_maxi_jour FROM circuit"
		logging.debug('SQL==> '+req)
		cursor.execute(req)
		circuits = []
		for (nom, parent, eclatable, puissance_maxi, energie_maxi_jour)  in cursor:
			circuits.append(circuit_mesure(nom, parent, eclatable, puissance_maxi, energie_maxi_jour))
		logging.debug("get_circuits_mesures : %s" % str(circuits))
		cursor.close()
		return circuits
		
	@DB.unError
	def get_electric_devices(self):
		'''Extrait les circuits electric_device
		'''
		cursor = self.db.cursor()
		req = "SELECT  nom, parent, puissance_maxi, energie_maxi_jour, puissance_typique FROM electric_device"
		logging.debug('SQL==> '+req)
		cursor.execute(req)
		circuits = []
		for (nom, parent, puissance_maxi, energie_maxi_jour, puissance_typique)  in cursor:
			logging.debug("electric_device : %s puissance_typique = %s" % (nom, puissance_typique))
			circuits.append(electric_device(nom, parent, puissance_maxi, energie_maxi_jour, puissance_typique))
		logging.debug("get_electric_devices : %s" % str(circuits))
		cursor.close()
		return circuits
	
	#@DB.unError
	def get_conso(self, nom_circuit, date_debut):
		'''renvoie la conso d'un circuit à une date de début
		'''
		cursor = self.db.cursor()
		req = u'SELECT  Id, date_debut, date_fin, energie, puissance, type_horaire FROM consos_electrique WHERE circuit = %s AND date_debut = %s;'
		logging.debug('SQL==> '+ req )
		logging.debug('nom_circuit : %s, date_debut : %s' % (nom_circuit, date_debut.__repr__()))
		cursor.execute(req, (nom_circuit, date_debut))
		results = cursor.fetchall()
		cursor.close()
		if len(results)==0:
			logging.warning("Pas de conso pour le circuit : %s at %s" % (nom_circuit, date_debut))
			return None
		elif len(results)>1:
			logging.warning("Trop de conso pour le circuit : %s at %s" % (nom_circuit, date_debut))
			logging.warning("Seule la premiere conso est prise en compte")
		return results[0]
	
	@DB.unError
	def get_last_power(self, nom_circuit):
		''' Renvoie la dernière puissance d'un circuit
		'''
		cursor = self.db.cursor()
		req = u'SELECT  puissance FROM consos_electrique WHERE circuit=%s AND date_debut = (SELECT MAX(date_debut) FROM consos_electrique WHERE circuit=%s)'
		logging.debug('SQL==> '+ req )
		logging.debug('nom_circuit : %s' % (nom_circuit))
		cursor.execute(req, (nom_circuit))
		power = cursor.fetchone()
		return power
	
	@DB.unError
	def get_last_date(self, nom_circuit):
		''' Renvoie la derniere date a laquelle la puissance du circuit a ete enregistree
		'''
		cursor = self.db.cursor()
		req = u"SELECT MAX(date_debut) FROM consos_electrique WHERE circuit = '%s'" % nom_circuit
		logging.debug('SQL==> '+ req )
		cursor.execute(req)
		result = cursor.fetchone()
		try:
			return result[0]
		except IndexError:
			return None
	
	@DB.unError
	def get_last_date_power_under_than(self, nom_circuit, power_level):
		''' Renvoie la derniere date a laquelle la puissance du circuit a ete en dessous de power_level
		'''
		cursor = self.db.cursor()
		req = u'SELECT MAX(date_debut) FROM `consos_electrique` WHERE circuit = %s and puissance < %s'
		logging.debug('SQL==> '+ req )
		logging.debug('nom_circuit : %s, powel_level : %s' % (nom_circuit, power_level))
		cursor.execute(req, (nom_circuit, power_level))
		result = cursor.fetchone()
		try:
			return result[0]
		except IndexError:
			return None
	
	@DB.unError
	def get_average_power(self, nom_circuit, duration = 3600):
		'''Renvoie la puissance moyenne pendant la période.
			- nom_circuit	:	nom du circuit electriques
			- duration		:	durée en seconde
		'''
		cursor = self.db.cursor()
		date_now = datetime.datetime.now()
		date_init = date_now - datetime.timedelta(seconds=duration)
		req = u'SELECT SUM(energie) FROM consos_electrique WHERE circuit = %s and date_debut BETWEEN %s and %s'
		logging.debug('SQL==> '+ req )
		logging.debug('nom_circuit : %s, date_debut : %s / %s' % (nom_circuit, date_init, date_now))
		cursor.execute(req, (nom_circuit, date_init, date_now))
		result = cursor.fetchone()
		try:
			return result[0] / duration * 3600
		except IndexError, TypeError:
			return 0
	
	@DB.unError
	def get_not_eclate_consos(self, nom):
		'''Renvoie les consommations qui ne sont pas notées éclatées
		'''
		cursor = self.db.cursor()
		req = "SELECT  Id, date_debut, date_fin, energie, puissance, type_horaire FROM `consos_electrique` " \
				+ "WHERE circuit = '%s' AND analyse = false" % nom
		logging.debug('SQL==> '+req+'->'+nom)
		cursor.execute(req)
		results = cursor.fetchall()
		cursor.close()
		return results
	
	@DB.unError
	def set_analyse(self, Id, analyse):
		''' Update the Id conso_electrique with flag analyse
		'''
		cursor = self.db.cursor()
		req = "UPDATE consos_electrique SET analyse = %s WHERE Id = %s" % (analyse, Id)
		logging.debug('SQL==> '+req)
		cursor.execute(req)
		self.db.commit()
		cursor.close()

class DBError(Exception): 
	def __init__(self, message):
		self.message = message
	def __str__(self):
		return self.message



#!/usr/bin/env python
# -*- coding:utf-8 -*
'''
	Fichier de configuration pour FERG
	
	Déclaration de l'installation
	
	
'''
#Pour travailler sur les sources
import sys
sys.path.insert(0,'../FGPIO')
sys.path.insert(0,'../FUTIL')

try:
	from FGPIO.mcp300x_hspi_io import *
	from FGPIO.SCT013_v_io import *
	from FGPIO.relay_io import *
	from FGPIO.lcd_i2c_io import *
	from FGPIO.lum_capteur_io import *
	from FGPIO.buzz_io import *
	from FGPIO.i2c_device_io import *
	from FERG.compteur_edf import *
except ImportError: 
	pass
	
from FERG.tempeDB import *
from FERG.installation import *
from FERG.circuit import *
import FUTIL.mails

def get_installation(physical_device = True):
	'''Renvoie une installation configurée
		si physical_device == False : on ne déclara pas les capteurs, mais uniquement le "squelette" de l'installation
	'''
	#
	#########################################################
	#														#
	#				BASE DE DONNEES							#
	#														#
	#########################################################
	#
	# Base de données pour les consos electriques
	db_elect = conso_electriqueDB( \
					db_name = 'tempeDB', \
					user = 'fred', \
					passwd='achanger', \
					host = '192.168.10.10', \
					mail = FUTIL.mails.gmail(gmail_account='monmail@gmail.com', gmail_pwd='xxxxx'))
	#
	#########################################################
	#														#
	#				MATERIEL								#
	#														#
	#########################################################
	#
	nom_compteur = 'General'

	# Nano pc : un Raspberry (ou pcduino)
	pc = rpiduino_io()
		
	if physical_device:		
		# Nano pc : un Raspberry (ou pcduino)
		pc = rpiduino_io()
		#
		#Le capteur heure creuse est réalisé avec un detecteur de tension
		# Il fonctionne comme un bouton poussoir vers la masse.
		capteur_heure_creuse = bt_io(pc.bcm_pin(24))
		#
		#Convertisseur Analogique/Numérique pour lecture analogique sur Rpi
		mcp3008 = mcp3008_hspi_io()
		#
		# Atmega 328 pour detection bande noire sur roue du compteur edf connecté en mode i2c
		atmega = i2c_device_io(pc=pc, addr = 0x12)
		#
		# Detecteur optique pour detection bande noire sur roue du compteur edf
		#capteur_roue = qrd1114_analog_io(mcp3008.pin[0])
		#
		# Led qui s'allume quand la bande noire de la roue est détectée
		led = led_io(pc.bcm_pin(16))
		#
		#
		# Led qui s'allume en heures Creuses
		#led = led_io(pc.bcm_pin(20))
		#
		# Vieux compteur EDF
		compteur = compteur_edf( \
					#capteur = capteur_roue, \
					micro_controleur = atmega, \
					capteur_hc = capteur_heure_creuse, \
					led = led, \
					intensity_max = 45, \
					energy_tr = 2.5, \
					nb_tours_mesure = 10)
		#
		# Les tores pour mesurer le courant sur différents circuits
		tores = {'circuit_1': SCT013_v_io(mcp3008.pin[1], Imax = 30.0, Vmax = 1.0), \
					'circuit_ce': SCT013_v_io(mcp3008.pin[2], Imax = 30.0, Vmax = 1.0), \
					'circuit_frigo': SCT013_v_io(mcp3008.pin[3], Imax = 30.0, Vmax = 1.0), \
					'circuit_ext_haut': SCT013_v_io(mcp3008.pin[4], Imax = 30.0, Vmax = 1.0), \
					'circuit_frigo_cuisine': SCT013_v_io(mcp3008.pin[5], Imax = 30.0, Vmax = 1.0), \
					'circuit_divers': SCT013_v_io(mcp3008.pin[6], Imax = 30.0, Vmax = 1.0) \
					}
		#
		#Relais de delestage du chauffe eau
		deleste_chauffe_eau = relay_io(pc.bcm_pin(23))
		deleste_chauffe_eau.off() # off : circuit fermé (pas de délestage)
		#
		#Buzzer pour alertes
		#buzzer = buzz_io(pc.bcm_pin(25))
		
	else:
		tores = {'circuit_1': None, \
					'circuit_ce': None, \
					'circuit_frigo': None, \
					'circuit_ext_haut': None, \
					'circuit_frigo_cuisine': None, \
					'circuit_divers': None \
					}
		compteur = None
		deleste_chauffe_eau = None
		#buzzer = None
	
	buzzer = buzz_io(pc.bcm_pin(25))
	#########################################################
	#														#
	#				LES CIRCUITS							#
	#														#
	#########################################################
	#
	
	# Circuit général avec Compteur EDF de type vieux!
	circuit_0 = circuit_general( \
		nom = nom_compteur, \
		puissance_maxi = 10000,
		energie_maxi_jour = 50000, \
		compteur = compteur)

		#Circuits secondaires mesurés
	
	circuit_1 = circuit_mesure( \
			nom = 'circuit_1', \
			parent = circuit_0, \
			eclatable = True, \
			puissance_maxi = 2000, \
			energie_maxi_jour = 7000, \
			compteur = tores['circuit_1'])

	circuit_ce = circuit_mesure( \
			nom = 'circuit_ce', \
			parent = circuit_1, \
			puissance_maxi = 1500, \
			energie_maxi_jour = 7000, \
			compteur = tores['circuit_ce'], \
			delest = deleste_chauffe_eau, \
			puissance_typique = 1300)

	circuit_frigo_cuisine = circuit_mesure( \
			nom = 'circuit_frigo_cuisine', \
			parent = circuit_0, \
			eclatable = True, \
			puissance_maxi = 2000, \
			energie_maxi_jour = 7000, \
			compteur = tores['circuit_frigo_cuisine'])

	circuit_frigo = circuit_mesure( \
			nom = 'circuit_frigo', \
			parent = circuit_frigo_cuisine, \
			eclatable = True, \
			puissance_maxi = 350, \
			energie_maxi_jour = 3000, \
			compteur = tores['circuit_frigo'])
	
	circuit_ext_haut = circuit_mesure( \
			nom = 'circuit_ext_haut', \
			parent = circuit_0, \
			eclatable = True, \
			puissance_maxi = 2000, \
			energie_maxi_jour = 7000, \
			compteur = tores['circuit_ext_haut'])
			
	circuit_divers = circuit_mesure( \
			nom = 'circuit_divers', \
			parent = circuit_0, \
			eclatable = True, \
			puissance_maxi = 2000, \
			energie_maxi_jour = 7000, \
			compteur = tores['circuit_divers'])
			
	circuits_mesures = [circuit_0, circuit_1, circuit_ce, circuit_frigo, circuit_ext_haut, circuit_frigo_cuisine, circuit_divers]
		
	# Circuit physique (ou appareil) non mesuré
	
	congelateur = electric_device( \
			nom = 'Congelateur', \
			parent = circuit_frigo, \
			puissance_maxi = 130, \
			energie_maxi_jour = 1500, \
			puissance_typique = 110, \
			max_average_power = 0.80*110, \
			duration_average_power = 2*3600,\
			fonctionement_max = 3600 # Si la conso est > 50% de la puisance typique pendant 1 heure : alerte
			)
	
	refrigerateur = electric_device( \
			nom = 'Refrigerateur', \
			parent = circuit_frigo, \
			puissance_maxi = 230, \
			energie_maxi_jour = 2800, \
			puissance_typique = 190, \
			max_average_power = 0.75*190,\
			duration_average_power = 2*3600,\
			fonctionement_max = 2*3600 # Si la conso est > 50% de la puisance typique pendant 2 heures : alerte
			)
			
	buanderie = electric_device( \
			nom = 'Buanderie_chaufferie', \
			parent = circuit_1 \
			)
	
	cuisine = electric_device( \
			nom = 'Cuisine', \
			parent = circuit_frigo_cuisine \
			)
	
	# total_tores = electric_device( \
			# nom = 'Total tores', \
			# fils = [circuit_1, circuit_ext_haut, circuit_frigo_cuisine, circuit_divers] \
			# )
	
	electric_devices = [congelateur, refrigerateur, buanderie, cuisine]
	
	#
	#########################################################
	#														#
	#				L'INSTALLATION							#
	#														#
	#########################################################
	#
	return installation( \
				db = db_elect, \
				circuits_mesures = circuits_mesures,
				electric_devices = electric_devices, 
				seuil_delestage = 8000, \
				buzzer = buzzer)